@@ -1,5 +1,5 @@
 
 .onLoad <- function(libname=NULL, pkgname=prim)
 {
-  cat("prim 1.0.2 (2007)\n")
+  ##cat("prim 1.0.3 (2007)\n")
 }
